//
//  Part of the Java Shell 
//////////////////////////////////////////////////////////////////////////////
//
//   This is the basic "dump" command, it allows users to examine files they
//  have in the current directory.
//   This is a basic command that could be extended in a lot of ways.
//
//  To do:-
//   Allow users to specify directorys, and allow recursive listing.
//
//  Copyright (c) Steve Kemp 1998, skx@tardis.ed.ac.uk
//
//  To do:-
//   Usage of redirection, and piping, also allow input from the console
//  to reach the processes
//
//  Usage :-
//   java dump file1 file2 file3
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////

import java.io.*;

/*   A class capable of dumping a file, in a form reminiscent of the
   MS-DOS program debug.
     This class could be useful for those who wish to look at the textual
   internals of a program, or class file.

   @version 2
   @author Steve Kemp
 */
public class dump
{
  private DataInputStream bin;
  private byte oneline[] = new byte[ 16 ];
  private int readInLine;
  private int lineCount;
  private boolean okay;

  /** This is the method that starts the process of outputting a file<BR>
      First of all it determines whether its input is a file, if it is
      a directory then it returns immediately.<BR>
      Otherwise if its input file exists it can return a valid <B>Dump</B>
   object.
     @param file The name of the file it is supposed to dump
  */
  public dump( String file )
  { File actualFile = new File( file );
    try
    { if ( actualFile.isDirectory() )
      {  System.out.println( "Can't dump a directory." );
         this.okay = false;
         return;
      }

      if ( !actualFile.exists() )
      { System.out.println( "File not found" );
        this.okay = false;
        return;
      } 
      else
      {
        this.bin = new DataInputStream( new FileInputStream( file ) );
        this.lineCount = 0;
        this.readInLine = 0;
        this.okay = true;
      }
    }
    catch( Exception e )
    { System.out.println( "Error in dump.dump( String file ) " + e );
    }
  }

  /** This is the method that does the actual dumping of a file.<BR>
      It loops reading, and printing lines until it can't anymore
   */
  public void dumpFile()
  { try
    { while( true )
      { this.readInLine = readLine();
        lineCount++;
        if ( this.readInLine > 0 )
          printLine( this.readInLine );
        else
          break;
      }
      bin.close();
    }
    catch( Exception e )
    { System.out.println( "Error in dump.dump() " + e );
    }
  }


 /** 
   Read no more that 16 bytes into an internal array, and return an int
   holding the number of Bytes actually read
   @return The number of bytes actually read
  */
  public int readLine()
  { 
    try
    { int i;
      i = bin.read( oneline );
      return i;
    }
    catch( Exception e )
    { System.out.println( "Error in dump.readLine() " + e );
    }
    return 16;
  }

 //Print the first i items of the array oneLIne
 private void printLine( int i )
 { System.out.print( lineCount + ": " );
   String temp;
   String output;
   String hexpart = new String( "" );
   String stringpart = new String( "" );

   for ( int j = 0; j < i; j++ )
   { //System.out.print( this.oneline[ j ] + " " );
     temp = oneline[ j ] + "";
     Integer value = new Integer( temp );
     output = value.toHexString( value.intValue() );
     if ( output.length() == 1 )
      output = "0" + output;
     if ( output.length() > 2 )
     { hexpart += ( output.charAt( output.length() -2 ) );
       hexpart += ( output.charAt( output.length() -1 ) + " "  );
     }
     else
       hexpart += ( output + " " );     
   }

   stringpart = " ";
   for ( int j = 0; j < i; j++ )
   { char c = 'c';
     c = (char)oneline[ j ];
     Character d = new Character( c );
     if ( d.isLetterOrDigit( c ) )
       stringpart += c + "";
     else
       stringpart += " ";
   }

   System.out.print( hexpart );
   System.out.print( stringpart );
   System.out.print( "\n" );
 }


 /** This is the driver for this class, it treats each of its arguments
    as a filename, and passes them one by one to be printed.
<BR>
     If the input file was valid then the dump object it is returned
    can be used, so this method goes on to print out the contents ofthe
    file
    @see dump
    @see dump
 */
 public static void main( String args[] )
 { // Check to see if we have at least one argument, if not exit
   if ( args.length < 1 )
   { System.out.println( "java dump file1 file2 file3 ... filen");
   }
   else
   {
   // Otherwise process each file
     for ( int i = 0; i < args.length; i++ )
     { dump d = new dump( args[i] );
       if ( d.okay )
         d.dumpFile();
     }
   }
 }

}
